searchData={"items":[{"type":"task","title":"mix fireside.install","doc":"Installs a Fireside component.","ref":"Mix.Tasks.Fireside.Install.html"},{"type":"task","title":"Args - mix fireside.install","doc":"mix fireside.install `component`","ref":"Mix.Tasks.Fireside.Install.html#module-args"},{"type":"task","title":"Supported formats - mix fireside.install","doc":"* `component@path:../path/to/component` - The component will be installed from the specified path.","ref":"Mix.Tasks.Fireside.Install.html#module-supported-formats"},{"type":"task","title":"mix fireside.update","doc":"Updates a Fireside component.","ref":"Mix.Tasks.Fireside.Update.html"},{"type":"task","title":"Args - mix fireside.update","doc":"mix fireside.update `component`","ref":"Mix.Tasks.Fireside.Update.html#module-args"},{"type":"task","title":"Supported formats - mix fireside.update","doc":"* `component` - The component to be updated","ref":"Mix.Tasks.Fireside.Update.html#module-supported-formats"},{"type":"extras","title":"Home","doc":"# Fireside\n\nFireside is an Elixir library to install and maintain\nself-contained application logic in your existing Elixir application with smart\ncode generation and Abstract Syntax Tree (AST) hashing.\nSince the project is still in its early stages, its core functionality may\nchange unexpectedly. It uses [Igniter](https://hexdocs.pm/igniter) under\nthe hood to orchestrate code generation and modifications, which means that\nit's possible to hook into it and add advanced generation steps such as\nadding configuration to `config/config.exs`, adding a new child to the\napplication supervision tree, etc. Igniter itself uses\n[`Sourceror.Zipper`](https://hexdocs.pm/sourceror/zippers.html),\nso it is possible to do anything that Elixir's metaprogramming tools support,\nbut is probably unnecessary in most cases.","ref":"readme.html"},{"type":"extras","title":"Installation - Home","doc":"```elixir\ndef deps do\n  [\n    {:fireside, \"~> 0.0.1\", only: :dev, runtime: false}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"What is a Fireside component? - Home","doc":"A Fireside component is a self-contained piece of application logic\nwith a `fireside.exs` definition in its root. To create a Fireside component,\ncreate a new Mix project with `mix new my_app --sup` and define your\napplication logic, i.e. core functionality and tests. Then, for example, if\nyour Fireside component adds a Products context, you may have the following\n`fireside.exs` definition:\n\n```elixir\ndefmodule MyApp.FiresideComponent do\n  def config do\n    %{\n      lib: [\n        \"lib/my_app/products.ex\",\n        \"lib/my_app/products/**/*.{ex,exs}\"\n      ],\n      overwritable: [\"lib/my_app/products/definitions.ex\"],\n      tests: [\n        \"test/my_app/**/*_test.{ex,exs}\"\n      ],\n      test_supports: [\n        \"test/support/my_app/products_factory.ex\"\n      ]\n    }\n  end\n\n  def setup(igniter) do\n    # Add custom Igniter logic here, if necessary. It will be run immediately\n    # after the files above are imported.\n    igniter\n  end\nend\n```\n\nNow, it is possible to \"install\" this application into an existing Elixir\nproject with Fireside. The Fireside installer will take the following steps:\n\n1. Look at `mix.exs` of the Fireside component and install all its dependencies\n   as well as their Igniter installers. What this means is that, for example, if\n   you are adding a Fireside component that uses [`Ash`](https://hexdocs.pm/ash)\n   and [`AshPostgres`](https://hexdocs.pm/ash_postgres), their individual Igniter\n   installers will be run, setting up everything along the way, such as the\n   `Repo` module, required `config.exs` configuration, testing utilities, etc.\n2. Import the source code from the paths in `MyApp.FiresideComponent.config/0`.\n3. Run `MyApp.FiresideComponent.setup/1` for optional custom code modification.\n4. Replace `MyApp` across the entire project with the prefix of your\n   application.\n5. Calculate the hash of each imported file (except the files listed as\n   `overwritable`) and add it to its top, with a note that the file should not\n   be changed manually.\n\nAt this point, the component should become a native part of the existing Elixir\napplication. If its version is updated remotely, Fireside will be able to\nreplace all relevant parts as long as the hashes of their AST do not differ\nfrom their originals by running `fireside.update my_app`. If at any point the\ncomponent is no longer necessary, it can be removed with\n`mix fireside.delete my_app`. If it is no longer sufficient\nand needs to be extended/customized, it should be unlocked with\n`mix fireside.unlock my_app` and all the files will become as if they were\nnever a Fireside component to begin with. If they are modified without\nunlocking, Fireside will no longer be able to update them in the future,\nand the generated files will contain a no-longer-useful notice that they should\nnot be modified.","ref":"readme.html#what-is-a-fireside-component"},{"type":"extras","title":"Example application - Home","doc":"To see an example Fireside component, check out [Shopifex](https://github.com/ibarakaiev/shopifex),\na component that provides the backbone for an e-commerce online store.\nIf you want to test it out, create a new Mix project with\n`mix new fireside_playground --sup`, clone Shopifex alongside it (in the same\nparent directory), add `{:fireside, \"~> 0.0.1\"}` to `mix.exs` of\n`FiresidePlayground` and then run\n`mix fireside.install shopifex@path:../shopifex`.","ref":"readme.html#example-application"},{"type":"extras","title":"Tasks - Home","doc":"","ref":"readme.html#tasks"},{"type":"extras","title":"Currently supported tasks - Home","doc":"- **`fireside.install {app_name}@path:..`**: installs an existing\n  Fireside component under specified path.\n- **`fireside.update {app_name}`**: updates an installed Fireside component\n  to reflect all changes upstream.","ref":"readme.html#currently-supported-tasks"},{"type":"extras","title":"Planned tasks - Home","doc":"- **`fireside.init`**: creates a `fireside.exs` in the root of the project.\n  Will potentially include smart logic to guess its contents as well. This\n  should be run only when developing a component.\n- **`fireside.install {app_name}@github:..`** or **`fireside.install {app_name}@git:..`**:\n  installs a Fireside component from a Git reference or from Github.\n- **`fireside.install {app_name}@{version}`**: installs a Fireside component\n  from the Fireside directory (TBD).\n- **`fireside.unlock {app_name}`**: removes the lock from the provided\n  component. Specifically, it will remove all hash information from the\n  associated files and from `config/fireside.exs`. Once unlocked, a component\n  becomes a regular part of the source code and Fireside will no longer be able\n  to identify, track, and update it.\n- **`fireside.install .. --unlocked`**: same as **`fireside.install`** but\n  without locking the component. This can be useful if there are no plans to\n  fetch changes from the upstream.\n- **`fireside.delete {app_name}`**: completely removes the installed component\n  from the project. Note: installed dependencies will remain in `mix.exs` and\n  will need to be manually removed.","ref":"readme.html#planned-tasks"},{"type":"extras","title":"Future direction - Home","doc":"If this project proves to be useful to people other than myself, I might\ncreate a centralized directory for Fireside components, similar to Hex,\nwhere individual developers can publish and maintain their components. This\nwould hopefully create a helpful ecosystem of building blocks which other\ndevelopers can use to rapidly iterate their app with. In principle, these\ncomponents could be monetized as well.","ref":"readme.html#future-direction"},{"type":"extras","title":"Why does Fireside exist? - Home","doc":"There are certain application logic components that can (and maybe should)\nbe centralized and yet don't necessarily make sense to be a library\n(i.e. if they depend on a database). Typically, they become a SaaS, either\nself-hosted or paid for as a service, and require setting up communication\nchannels, adding unnecessary complexity.\n\nFireside hopes to provide an alternative by making it as easy as possible to\nreuse application logic by _embedding_ it within your Elixir monolith and still\nhave all the benefits like version upgrades without additional engineering\noverhead.\n\nFireside's implementation in Elixir is possible thanks to\n[@ZachDaniel](https://github.com/zachdaniel)'s ongoing work on\n[Igniter](https://hexdocs.pm/igniter), which powers most of Fireside, allowing\nfor smart, composable code generation and modification.","ref":"readme.html#why-does-fireside-exist"},{"type":"extras","title":"Why does Fireside not exist in other languages? - Home","doc":"Elixir/Phoenix/OTP is one of the few standardized ecosystems with common\nconventions that allows creating an arbitrarily large distributed monolith with\na solid foundation\n([a slide from Sasa Juric's talk](https://www.reddit.com/r/elixir/comments/gpdlp4/the_more_i_learn_about_elixir_the_more_i_realize/)\ncomes to mind). This allows writing code that can easily be reused across teams\nand companies (compare this to the JavaScript or Python ecosystem with a\nmillion different options to just install a package...).\n\nAdditionally, Elixir is an immutable language that supports metaprogramming.\nIn its closest spiritual relative and its ecosystem, Ruby and Ruby on Rails,\nFireside would probably not be possible due to mutability concerns and lack of\nmetaprogramming.","ref":"readme.html#why-does-fireside-not-exist-in-other-languages"},{"type":"extras","title":"Changelog","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [v0.0.3](https://github.com/ash-project/igniter/compare/v0.0.1...v0.0.2) (2024-08-01)","ref":"changelog.html"},{"type":"extras","title":"Improvements - Changelog","doc":"- Add Changelog to docs","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- support no `overwritable` in `fireside.install`.\n\n## [v0.0.2](https://github.com/ash-project/igniter/compare/v0.0.1...v0.0.2) (2024-07-21)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"- replace OTP application name in addition to module prefixes\n  (i.e. :my_app in addition to MyApp)","ref":"changelog.html#improvements"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}